@@ -30,15 +30,14 @@ QList<Profile::Ptr> ProfileGroup::profiles() const
 
 void ProfileGroup::updateValues()
 {
-    const PropertyInfo* properties = Profile::DefaultPropertyNames;
+    const PropertyInfo *properties = Profile::DefaultPropertyNames;
     while (properties->name != nullptr) {
         // the profile group does not store a value for some properties
         // (eg. name, path) if even they are equal between profiles -
         //
         // the exception is when the group has only one profile in which
         // case it behaves like a standard Profile
-        if (_profiles.count() > 1 &&
-                !canInheritProperty(properties->property)) {
+        if (_profiles.count() > 1 && !canInheritProperty(properties->property)) {
             properties++;
             continue;
         }
@@ -58,7 +57,7 @@ void ProfileGroup::updateValues()
     }
 }
 
-void ProfileGroup::setProperty(Property p, const QVariant& value)
+void ProfileGroup::setProperty(Property p, const QVariant &value)
 {
     if (_profiles.count() > 1 && !canInheritProperty(p)) {
         return;
