@@ -103,7 +103,14 @@ public:
      */
     void toggleMaximizeCurrentTerminal();
 
-    void handleMinimizeMaximize(bool maximize);
+    /**
+     * Can be called on any ViewSplitter to find the top level splitter and ensure
+     * the active display isn't maximized. Do nothing if it's not maximized.
+     *
+     * Useful for ViewManager and TabbedViewContainer when removing or adding a
+     * display to the hierarchy so that the layout is reflowed correctly.
+     */
+    void clearMaximized();
 
     /** returns the splitter that has no splitter as a parent. */
     ViewSplitter *getToplevelSplitter();
@@ -149,7 +156,7 @@ Q_SIGNALS:
 
 private:
     /** recursively walks the object tree looking for Splitters and
-     * TerminalDisplays, hidding the ones that should be hidden.
+     * TerminalDisplays, hiding the ones that should be hidden.
      * If a terminal display is not hidden in a subtree, we cannot
      * hide the whole tree.
      *
@@ -157,6 +164,9 @@ private:
      */
     bool hideRecurse(TerminalDisplay *currentTerminalDisplay);
 
+    /** other classes should use clearmMaximized() */
+    void handleMinimizeMaximize(bool maximize);
+
     void updateSizes();
     bool m_terminalMaximized = false;
     bool m_blockPropagatedDeletion = false;
